//
//  NDKCVSParser.m
//  NetDataKit
//
// Copyright (c) 2012, Paul C. Hoehne
// All rights reserved.
//
// Redistribution and use in source and binary forms, with 
// or without modification, are permitted provided that the 
// following conditions are met:
//
// Redistributions of source code must retain the above 
// copyright notice, this list of conditions and the 
// following disclaimer.
// 
// Redistributions in binary form must reproduce the above 
// copyright notice, this list of conditions and the 
// following disclaimer in the documentation and/or other 
// materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
// DAMAGE.
//

#import "NDKCharacterDelimitedParser.h"
#import "NDKDataSource.h"
#import "NDKErrorReport.h"

@implementation NDKCharacterDelimitedParser

@synthesize delimiter;

-(id) init {
    self = [super init];
    
    [self setDelimiter:','];
    
    return self;
}

-(NSMutableArray*) stripBlanks: (NSArray*)vals {
    NSMutableArray* result = [NSMutableArray array];
    for(NSUInteger i = 0; i < [vals count]; i++) {
        [result addObject:[[vals objectAtIndex:i] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]]];
    }
    [result removeObject:@""];
    return result;
}

-(NSArray*) parse: (NSData*)data forSource: (NDKDataSource*)source {
    NSMutableArray* result = nil;
    NSString* unparsed = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    NSCharacterSet* eolSet = [NSCharacterSet newlineCharacterSet];
    NSCharacterSet* delimiterSet = [NSCharacterSet characterSetWithCharactersInString:
                                    [NSString stringWithFormat:@"%c", [self delimiter]]];
    NSArray* lines = [unparsed componentsSeparatedByCharactersInSet:eolSet];
    NSArray* keys = nil;
    for (NSString* line in lines) {
        if(result == nil) {
            result = [[NSMutableArray alloc] init];
        }
        if([line length] > 0) {
            NSMutableArray* subElements = [NSMutableArray arrayWithArray:[line componentsSeparatedByCharactersInSet:delimiterSet]];
            subElements = [self stripBlanks:subElements];
            
            if(keys == nil) { 
                keys = subElements;
            } else {
                if([keys count] != [subElements count]) {
                    NSString* errorString = [NSString stringWithFormat:@"There are %d keys and only %d elements in this line of the input file", [keys count], [subElements count]];
                    NSDictionary* userInfo = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:keys, subElements, nil] forKeys: [NSArray arrayWithObjects:@"keys", @"values", nil]];
                    
                    NDKErrorReport* report = [[NDKErrorReport alloc] initWithSeverity:NDKWarningSeverity andMssage: errorString andInfo: userInfo];
                    [source addReport:report inPhase:NDKParsingPhase];
                } else {
                    [result addObject: [NSDictionary dictionaryWithObjects:subElements forKeys:keys]];
                }
            }
            
        }
    }
    
    
    return result;
}

@end
