//
//  NDKDataSource.h
//  NetDataKit
//
// Copyright (c) 2012, Paul C. Hoehne
// All rights reserved.
//
// Redistribution and use in source and binary forms, with 
// or without modification, are permitted provided that the 
// following conditions are met:
//
// Redistributions of source code must retain the above 
// copyright notice, this list of conditions and the 
// following disclaimer.
// 
// Redistributions in binary form must reproduce the above 
// copyright notice, this list of conditions and the 
// following disclaimer in the documentation and/or other 
// materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
// DAMAGE.
//

#import <Foundation/Foundation.h>
#import "NDKParser.h"
#import "NDKDataLocator.h"
#import "NDKFieldMapper.h"

enum {
    NDKNoError = -1,
    NDKNoDataLocatorError = 0,
    NDKNoDataParserError = 1,
    NDKDataNotFoundError = 2,
    NDKParseFailedError = 3,
    NDKMappingFailedError = 4
};

enum {
    NDKLocatingPhase = 0,
    NDKParsingPhase = 1,
    NDKMappingPhase = 2
};

@class NDKErrorReport;

@interface NDKDataSource : NSObject {
    NSMutableDictionary* errorReports; // By phase
    
}

@property (strong) id<NDKParser> parser;
@property (strong) id<NDKDataLocator> locator;
@property (strong) id<NDKFieldMapper> mapper;
@property (strong) NSString* errorMessage;
@property (assign) NSUInteger errorCode;

-(NSArray*) findData: (NSDictionary*)parameters;
-(void) addReport: (NDKErrorReport*)report inPhase: (NSUInteger)phase;
-(NSArray*) errorReportsForPhase: (NSUInteger)phase;

@end
