//
//  NDKDataSource.m
//  NetDataKit
//
// Copyright (c) 2012, Paul C. Hoehne
// All rights reserved.
//
// Redistribution and use in source and binary forms, with 
// or without modification, are permitted provided that the 
// following conditions are met:
//
// Redistributions of source code must retain the above 
// copyright notice, this list of conditions and the 
// following disclaimer.
// 
// Redistributions in binary form must reproduce the above 
// copyright notice, this list of conditions and the 
// following disclaimer in the documentation and/or other 
// materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
// DAMAGE.
//

#import "NDKDataSource.h"

@implementation NDKDataSource {
    
}

@synthesize locator, mapper, parser, errorCode, errorMessage;

-(id) init {
    self = [super init];
    
    [self setErrorCode:NDKNoError];
    errorReports = [[NSMutableDictionary alloc] init];
    
    return self;
}

-(NSArray*) findData:(NSDictionary *)parameters {
    NSArray* result = nil;
    NSData* rawData;
    if(locator == nil) { 
        [self setErrorCode:NDKNoDataLocatorError];
        [self setErrorMessage:@"No data locator was provided - unable to retrieve data"];
        return result;
    }
    
    rawData = [locator retrieve:parameters forSource:self];
    if(rawData == nil) {
        [self setErrorCode:NDKDataNotFoundError];
        [self setErrorMessage:@"Unable to locate data"];
        return result;
    }
    
    if(parser == nil) { 
        [self setErrorCode:NDKNoDataParserError];
        [self setErrorMessage:@"No data parser was provided - unable to parse data"];
        return result;
    }
    
    result = [parser parse:rawData forSource:self];
    if(result == nil) {
        [self setErrorCode:NDKParseFailedError];
        [self setErrorMessage:@"Unable to parse the data"];
        return result;
    }
    
    if(mapper != nil) {
        result = [mapper mapFields:result forSource:self];
    }
    if(result == nil) {
        [self setErrorCode:NDKMappingFailedError];
        [self setErrorMessage:@"Unable to map fields"];
        return result;
    }
    return result;
}

-(void) addReport: (NDKErrorReport*)report inPhase: (NSUInteger)phase {
    NSString* key = [NSString stringWithFormat:@"%d", phase];
    NSMutableArray* list = [errorReports valueForKey:key];
    if(list == nil) { 
        list = [[NSMutableArray alloc] init];
        [errorReports setValue:list forKey:key];
    }
    [list addObject:report];
}

-(NSArray*) errorReportsForPhase: (NSUInteger)phase {
    return [errorReports valueForKey:[NSString stringWithFormat:@"%d", phase]];
}


@end
