//
//  TestCharacterDelimitedParser.m
//  NetDataKit
//
// Copyright (c) 2012, Paul C. Hoehne
// All rights reserved.
//
// Redistribution and use in source and binary forms, with 
// or without modification, are permitted provided that the 
// following conditions are met:
//
// Redistributions of source code must retain the above 
// copyright notice, this list of conditions and the 
// following disclaimer.
// 
// Redistributions in binary form must reproduce the above 
// copyright notice, this list of conditions and the 
// following disclaimer in the documentation and/or other 
// materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
// DAMAGE.
//

#import "TestCharacterDelimitedParser.h"
#import "NDKCharacterDelimitedParser.h"
#import "NDKDataSource.h"
#import "NDKUrlLocator.h"

@implementation TestCharacterDelimitedParser

// All code under test must be linked into the Unit Test bundle
- (void)testParse
{
    NDKCharacterDelimitedParser* parser = [[NDKCharacterDelimitedParser alloc] init];
    NDKDataSource* dataSource = [[NDKDataSource alloc] init];
    NSData* sampleData = [@"a, b, c, d\n1, 2, 3, 4" dataUsingEncoding:NSUTF8StringEncoding];
    NSArray* data = [parser parse:sampleData forSource:dataSource];
    
    STAssertNotNil(data, @"There should be a data object");
    
    STAssertTrue([data count] == 1, @"There should be one row");
    STAssertTrue([[data objectAtIndex: 0] valueForKey:@"a"] != nil, @"Should have a value for 'a'");
    
    STAssertEqualObjects(@"1", [[data objectAtIndex: 0] valueForKey: @"a"], @"Should equal 1");
    STAssertEqualObjects(@"2", [[data objectAtIndex: 0] valueForKey: @"b"], @"Should equal 2");
    STAssertEqualObjects(@"3", [[data objectAtIndex: 0] valueForKey: @"c"], @"Should equal 3");
    STAssertEqualObjects(@"4", [[data objectAtIndex: 0] valueForKey: @"d"], @"Should equal 4");
    
}

-(void) testIncorrectCount
{
    NDKCharacterDelimitedParser* parser = [[NDKCharacterDelimitedParser alloc] init];
    NDKDataSource* dataSource = [[NDKDataSource alloc] init];
    NSData* sampleData = [@"a, b, c, d\n1, 2, 3" dataUsingEncoding:NSUTF8StringEncoding];
    NSArray* data = [parser parse:sampleData forSource:dataSource];
    
    STAssertNotNil(data, @"There should be a data object");
    
    STAssertTrue([data count] == 0, @"There should be no rows");
    
    STAssertEquals([[dataSource errorReportsForPhase: NDKParsingPhase] count], (NSUInteger)1, @"There should be an error report");
}

@end
