//
//  TestDataSource.m
//  NetDataKit
//
// Copyright (c) 2012, Paul C. Hoehne
// All rights reserved.
//
// Redistribution and use in source and binary forms, with 
// or without modification, are permitted provided that the 
// following conditions are met:
//
// Redistributions of source code must retain the above 
// copyright notice, this list of conditions and the 
// following disclaimer.
// 
// Redistributions in binary form must reproduce the above 
// copyright notice, this list of conditions and the 
// following disclaimer in the documentation and/or other 
// materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
// DAMAGE.
//

#import "TestDataSource.h"
#import "NDKDataSource.h"
#import "NDKDataLocator.h"
#import "NDKErrorReport.h"
#import "NDKUrlLocator.h"
#import "NDKCharacterDelimitedParser.h"

@implementation TestDataSource

// All code under test must be linked into the Unit Test bundle
- (void)testFindData
{
    NDKDataSource* dataSource = [[NDKDataSource alloc] init];
    NDKUrlLocator* urlLocator = [[NDKUrlLocator alloc] init];
    NDKCharacterDelimitedParser* parser = [[NDKCharacterDelimitedParser alloc] init];
    
    [urlLocator setUrl:[NSURL URLWithString:@"http://ichart.finance.yahoo.com/table.csv?s=AAPL&d=0&e=30&f=2012&g=d&a=8&b=7&c=1984&ignore=.csv"]];
    
    [dataSource setLocator:urlLocator];
    [dataSource setParser:parser];
    NSArray* results = [dataSource findData:nil];
    
    STAssertNotNil(results, @"The results should not be nil");
    STAssertTrue([results count] != 0, @"There should be items in the collection");
}

- (void)testNoLocator
{
    NDKDataSource* dataSource = [[NDKDataSource alloc] init];

    NSArray* results = [dataSource findData:nil];
    
    STAssertNil(results, @"Results should be nill");
    STAssertTrue([dataSource errorCode] == NDKNoDataLocatorError, @"The no locator error should be raised");
    STAssertNotNil([dataSource errorMessage], @"There should be an error message");
}

-(void) testAddReport
{
    NDKDataSource* dataSource = [[NDKDataSource alloc] init];
    [dataSource addReport:[[NDKErrorReport alloc] initWithSeverity:NDKWarningSeverity andMssage:@"This is just a warning"] inPhase:NDKLocatingPhase];
    [dataSource addReport:[[NDKErrorReport alloc] initWithSeverity:NDKDebugSeverity andMssage:@"This is a sever thing"] inPhase:NDKParsingPhase];
    [dataSource addReport:[[NDKErrorReport alloc] initWithSeverity:NDKInfoSeverity andMssage:@"This is just informational"] inPhase:NDKMappingPhase];
    
    STAssertTrue([[dataSource errorReportsForPhase: NDKParsingPhase] count] == (NSUInteger)1, @"There should be one error in he parsing phase");
    STAssertTrue([[dataSource errorReportsForPhase: NDKLocatingPhase] count] == (NSUInteger)1, @"There should be one locating error");
    STAssertTrue([[dataSource errorReportsForPhase: NDKMappingPhase] count] == (NSUInteger)1, @"There should be one mapping error");
    
}


@end
