//
//  TestSimpleFieldToClassMapper.m
//  NetDataKit
//
//
// Copyright (c) 2012, Paul C. Hoehne
// All rights reserved.
//
// Redistribution and use in source and binary forms, with 
// or without modification, are permitted provided that the 
// following conditions are met:
//
// Redistributions of source code must retain the above 
// copyright notice, this list of conditions and the 
// following disclaimer.
// 
// Redistributions in binary form must reproduce the above 
// copyright notice, this list of conditions and the 
// following disclaimer in the documentation and/or other 
// materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
// DAMAGE.
//
#import "TestSimpleFieldToClassMapper.h"

#import "NDKSimpleFieldToClassMapper.h"
#import "NDKDataSource.h"

@interface FooClass : NSObject {
}

@property (strong) NSString* name;
@property (strong) NSNumber* age;

@end

@implementation FooClass

@synthesize name, age;

@end

@implementation TestSimpleFieldToClassMapper

// All code under test must be linked into the Unit Test bundle
- (void)testSimpleMapping
{
    NDKSimpleFieldToClassMapper* mapper = [[NDKSimpleFieldToClassMapper alloc] init];
    NSMutableDictionary* mapping = [[NSMutableDictionary alloc] init];
    NDKDataSource* source = [[NDKDataSource alloc] init];
    
    [mapping setObject:@"name" forKey:@"otherName"];
    [mapping setObject:@"age" forKey:@"otherAge"];
    
    [mapper setObjectClass: [FooClass class]];
    [mapper setMapping: mapping];
    
    NSMutableArray* array = [[NSMutableArray alloc] init];
    [array addObject: [NSDictionary dictionaryWithObjectsAndKeys: 
                       @"Joe", @"otherName", 
                       [NSNumber numberWithInt:20], @"otherAge", nil]];
    [array addObject: [NSDictionary dictionaryWithObjectsAndKeys:
                       @"Jane", @"otherName",
                       [NSNumber numberWithInt:30], @"otherAge", nil]];
    
    
    NSArray* results = [mapper mapFields:array forSource:source];
    STAssertEquals((NSUInteger)2, [results count], @"There should be 2");
    
    STAssertEqualObjects([[results objectAtIndex:0] class], [FooClass class], @"It should be a FooClass object");
    STAssertEqualObjects(@"Joe", [[results objectAtIndex:0] name], @"The name should be 'Joe'");
    STAssertEqualObjects([NSNumber numberWithInt:20], [[results objectAtIndex:0] age], @"The age should be 20");
}

@end
