//
//  TestURLLocator.m
//  NetDataKit
//
// Copyright (c) 2012, Paul C. Hoehne
// All rights reserved.
//
// Redistribution and use in source and binary forms, with 
// or without modification, are permitted provided that the 
// following conditions are met:
//
// Redistributions of source code must retain the above 
// copyright notice, this list of conditions and the 
// following disclaimer.
// 
// Redistributions in binary form must reproduce the above 
// copyright notice, this list of conditions and the 
// following disclaimer in the documentation and/or other 
// materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
// DAMAGE.
//

#import "TestURLLocator.h"
#import "NDKUrlLocator.h"
#import "NDKDataSource.h"
#import "NDKErrorReport.h"

@implementation TestURLLocator

- (void)testLocate
{
    NDKDataSource* dataSource = [[NDKDataSource alloc] init];
    NDKUrlLocator* locator = [[NDKUrlLocator alloc] init];
    [locator setUrl: [NSURL URLWithString:@"http://ichart.finance.yahoo.com/table.csv?s=AAPL&d=0&e=30&f=2012&g=d&a=8&b=7&c=1984&ignore=.csv"]];
    
    
    NSData* data = [locator retrieve:nil forSource:dataSource];
    STAssertNotNil(data, @"Expected to find data");
    
    STAssertTrue([dataSource errorCode] == NDKNoError , @"There should have been no error %d", [dataSource errorCode]);
    STAssertTrue([dataSource errorMessage] == nil, @"There should not be an error message %@", [dataSource errorMessage]);
}

- (void)testLocateFail 
{
    NDKDataSource* dataSource = [[NDKDataSource alloc] init];
    NDKUrlLocator* locator = [[NDKUrlLocator alloc] init];
    [locator setUrl: [NSURL URLWithString:@"http://ichart.finance.yahoo.com/table.csv?s=XAAPL&d=0&e=30&f=2012&g=d&a=8&b=7&c=1984&ignore=.csv"]];
    
    
    NSData* data = [locator retrieve:nil forSource:dataSource];
    STAssertNil(data, @"Not Expected to find data");
    
    NSArray* errors = [dataSource errorReportsForPhase: NDKLocatingPhase];
    
    STAssertTrue([errors count] > 0, @"There should be errors");
    NDKErrorReport* error = [errors objectAtIndex:0];
    STAssertNotNil(error, @"There shoudl be an error");
}

@end
